#!/usr/bin/python -u
#
# This creates a Qt resources XML file.
#
# Usage:
#
#  build-qt-resources.py output.qrc virtualpath1=realpath1 virtualpath2=realpath2 ... virtualpathN=realpathN
#
# "output.qrc" will be overwritten! Each argument after the output file
# consists of a virtual path, which defines the resource prefix, and a real
# path (i.e. an actual file system path), which sets the source file or
# directory. If the real path is a file, the virtual path sets the full
# virtual file name. If the real path is a directory, the directory is
# scanned recursively, and all files are added, using the virtual path as
# prefix.

import sys
import os
import stat
import io

if len(sys.argv) < 2:
  sys.exit(1)

result = "<RCC>\n"

def add_files(virtualpath, filepath):
  global result

  fstat = os.stat(filepath)
  if stat.S_ISDIR(fstat.st_mode):
    for item in os.listdir(filepath):
      add_files(os.path.join(virtualpath, item), os.path.join(filepath, item))
  else:
    dirname, fname = os.path.split(virtualpath)
    result += (('<qresource prefix=\"%s\">\n' +
                ' <file alias=\"%s\">%s</file>\n' +
                '</qresource>\n') % (dirname, fname, filepath))

for item in sys.argv[2:]:
  virtualpath, filepath = item.split("=", 1)
  add_files(virtualpath, filepath)

result += "</RCC>"

result = result.encode("utf8")

try:
  with open(sys.argv[1], "rb") as infile:
    if infile.read() == result:
      # They're the same -> prevent cmake from rebuilding resources.
      sys.exit(0)
except IOError:
  pass

outfile = open(sys.argv[1], "wb")
outfile.write(result)



#!/usr/bin/python
import subprocess, os, sys, shutil, Queue, fnmatch

def exec_cmd(args, env={}, supress_output=False):
    cmd = subprocess.Popen(args, stdout = subprocess.PIPE, stderr = subprocess.STDOUT, env = env)
    output = ''
    while True:
      out = cmd.stdout.read(1)
      if out == '' and cmd.poll() != None:
        break
      if out != '':
        if not supress_output:
          sys.stdout.write(out)
        output += out
    if cmd.wait() != 0:
      raise Exception("Command failed: \"%s\"" % " ".join(args), output)
    return output

binaries = {}

def install_libraries(binary):
  if binaries.has_key(binary): return []
  binaries[binary] = True
  other_binaries = []

  out = exec_cmd(["otool", "-L", binary], supress_output=True)
  for line in out.split('\n'):
    line = line.strip()
    if line.find("dylib") != -1 and (line.startswith('@executable_path') or line.find("libQt5") != -1):
      file = line.split('/')[-1].split(' ')[0]
      if line.startswith("@executable_path"):
        src = os.path.join(sys.argv[1], "dependencies/darwin-x86_64/lib", file)
      else:
        src = line.split(" (")[0]

      dst = os.path.join(sys.argv[2], "Konvergo.app", "Contents", "Frameworks", file)
      if os.path.exists(src):
        if (not os.path.exists(dst) or os.path.getsize(dst) != os.path.getsize(src)):
          print "-- Installing %s" % file
          shutil.copyfile(src, dst)

        other_binaries.append(src)

  return other_binaries

queue = Queue.Queue()

# Add the top-level binaries.
for b in ['Contents/MacOS/Konvergo']:
  queue.put(os.path.join(sys.argv[2], "Konvergo.app", b))

for root, dirs, files in os.walk(os.path.join(sys.argv[2], "Konvergo.app", "Contents", "Frameworks", "vlc")):
  for f in files:
    path = os.path.join(root, f)
    queue.put(path)

while queue.empty() == False:
  binary = queue.get()
  other_binaries = install_libraries(binary)
  for b in other_binaries:
    queue.put(b)


from pathlib import Path
import sys
import os
import argparse
from collections import deque
import subprocess
import shutil

def main(argv=tuple(sys.argv[1:])):
  arg_parser = argparse.ArgumentParser(description='Fix third party library paths in .app bundles.')
  arg_parser.add_argument('bundle', metavar='BUNDLE', type=str, nargs=1)
  arguments = arg_parser.parse_args(argv)

  bundle_path = Path(arguments.bundle[0])
  print(bundle_path)
  webengine_path = bundle_path / 'Contents' / 'Frameworks' / 'QtWebEngineCore.framework' / 'Helpers' / 'QtWebEngineProcess.app'
  link_path = webengine_path / 'Contents' / 'Frameworks'
  bin_to_fix = webengine_path / 'Contents' / 'MacOS' / 'QtWebEngineProcess'

  os.symlink('../../../../../../../Frameworks', link_path)

  result = subprocess.check_output(['otool', '-L', str(bin_to_fix.resolve())],stderr=subprocess.STDOUT).decode('utf-8')
  for dependency in result.splitlines():
    dependency = dependency.strip().lstrip()
    if dependency.startswith('/opt/homebrew') or dependency.startswith('/usr/local'):
      # cut off trailing compatibility string
      dependency_str = dependency.split(' (compatibility')[0].strip()
      dependency_framework_str = dependency_str.split('/lib')[1].strip()
      dependency_framework = Path(dependency_framework_str)
      target = f'@executable_path/../Frameworks{dependency_framework_str}'
      subprocess.run(['install_name_tool', '-id', target, bin_to_fix])
      subprocess.run(['install_name_tool', '-change', dependency_str, target, bin_to_fix])

if __name__ == '__main__':
  main()



from pathlib import Path
import sys
import argparse
from collections import deque
import subprocess
import shutil
import platform


def main(argv=tuple(sys.argv[1:])):
    arg_parser = argparse.ArgumentParser(description='Fix third party library paths in .app bundles.')
    arg_parser.add_argument('bundle', metavar='BUNDLE', type=str, nargs=1)
    arguments = arg_parser.parse_args(argv)

    bundle_path = Path(arguments.bundle[0])
    framework_path = bundle_path / 'Contents' / 'Frameworks'
    rpath_str = '/usr/local/lib'
    if platform.machine() == 'arm64':
        rpath_str = '/opt/homebrew/lib'
    framework_libs = set(file.name for file in framework_path.glob('*.dylib')).union(set(file.name for file in framework_path.glob('*.so')))
    libs_to_fix = deque()
    libs_to_fix.extend(file for file in bundle_path.glob('**/*.dylib'))
    libs_to_fix.extend(file for file in bundle_path.glob('**/*.so'))
    while libs_to_fix:
        lib = libs_to_fix.popleft()
        # find the dependencies of the library
        result = subprocess.check_output(['otool', '-L', str(lib.resolve())], stderr=subprocess.STDOUT).decode('utf-8')
        for dependency in result.splitlines():
            dependency = dependency.strip().lstrip()
            if dependency.startswith('/usr/local') or dependency.startswith('/opt/homebrew') or dependency.startswith('@rpath') or dependency.startswith('@loader_path'):
                # cut off trailing compatibility string
                dependency_str = dependency.split(' (compatibility')[0].strip()
                if dependency.startswith('@rpath'):
                    dependency = dependency_str.replace("@rpath", rpath_str)
                    dependency = Path(dependency)
                elif dependency.startswith('@loader_path'):
                    dependency = dependency_str.replace("@loader_path", rpath_str)
                    dependency = Path(dependency)
                    dependency_name = dependency.name
                    dependency = (Path(rpath_str) / dependency_name).resolve()
                else:
                    dependency = Path(dependency_str)

                # if somehow macdeployqt didn't copy the lib for us, we do a manual copy
                if dependency.name not in framework_libs:
                    shutil.copy(str(dependency.resolve()), str((framework_path / dependency.name).resolve()))
                    framework_libs.add(dependency.name)
                    # add the newly added library in to the to fix queue
                    libs_to_fix.append(framework_path / dependency.name)
                    print((framework_path / dependency.name).resolve())
                    print(f'Copied {dependency} to {framework_path / dependency.name}')

                # now we fix the path using install_name_tool
                target = f'@executable_path/../Frameworks/{dependency.name}'
                print(f'Fixing dependency {dependency_str} of {lib} to {target}')
                subprocess.run(['install_name_tool', '-id', target, lib])
                subprocess.run(['install_name_tool', '-change', dependency_str, target, lib])


if __name__ == '__main__':
    main()



#!/usr/bin/env python -u

# simple script that takes data on stdin and outputs bz2 compressed data
# just to avoid having to install bzip2.exe on windows platforms

import bz2, os, sys

bufsize=4096

if __name__ == "__main__":
  compress = bz2.BZ2Compressor()
  data = sys.stdin.read(bufsize)
  while data:
    cdata = compress.compress(data)
    if cdata:
      sys.stdout.write(cdata)
    data = sys.stdin.read(bufsize)

  cdata = compress.flush()
  if cdata:
    sys.stdout.write(cdata)
